#!/usr/bin/python

# Copyright (C) 2012-2015    ASTRON (Netherlands Institute for Radio Astronomy)
# P.O. Box 2, 7990 AA Dwingeloo, The Netherlands
#
# This file is part of the LOFAR software suite.
# The LOFAR software suite is free software: you can redistribute it and/or
# modify it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# The LOFAR software suite is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.    See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with the LOFAR software suite. If not, see <http://www.gnu.org/licenses/>.

# $Id:  $
import unittest, datetime, mock

# you might need to install mock, mysql.connector(from Oracle), testing.mysqld, mock, coverage, 
# lxml, xmljson, django, djangorestframework, djangorestframework_xml, python-ldap, six, qpid, mllib
# using things like sudo pip install <package>

from lofar.sas.resourceassignment.taskprescheduler.taskprescheduler import TaskPrescheduler
from lofar.sas.resourceassignment.taskprescheduler.taskprescheduler import calculateCobaltSettings
from lofar.sas.resourceassignment.taskprescheduler.taskprescheduler import resourceIndicatorsFromParset
from lofar.sas.resourceassignment.taskprescheduler.taskprescheduler import cobaltOTDBsettings
from lofar.sas.resourceassignment.taskprescheduler.taskprescheduler import main as PreschedulerMain


class TestingTaskPrescheduler(TaskPrescheduler):
    def __init__(self, otdbrpc, momrpc, radbrpc):
        # super gets not done to be able to insert mocks as early as possible otherwise the RPC block unittesting
        self.otdbrpc = otdbrpc
        self.momquery = momrpc
        self.radbrpc = radbrpc

class PreschedulerTest(unittest.TestCase):
    # No __init__ because that confuses unittest.main()
    
    def reset_specification_tree(self, otdb_id, mom_id, future_start_time, future_stop_time):
        self.pipeline_specification_tree = {
            u'ObsSW.Observation.DataProducts.Output_InstrumentModel.enabled': False,
            u'ObsSW.Observation.stopTime': future_stop_time,
            u'ObsSW.Observation.VirtualInstrument.stationList': [],
            u'ObsSW.Observation.DataProducts.Input_CoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Output_CoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Output_SkyImage.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_Correlated.skip': [0, 0, 0, 0],
            u'ObsSW.Observation.antennaSet': u'LBA_INNER',
            u'ObsSW.Observation.nrBitsPerSample': u'16',
            u'ObsSW.Observation.ObservationControl.PythonControl.LongBaseline.subbandgroups_per_ms': u'1',
            u'ObsSW.Observation.DataProducts.Output_IncoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_IncoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_Correlated.enabled': True,
            u'ObsSW.Observation.DataProducts.Output_Pulsar.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_CoherentStokes.skip': [],
            u'ObsSW.Observation.ObservationControl.PythonControl.DPPP.demixer.demixtimestep': u'10',
            u'Version.number': u'33774',
            u'ObsSW.Observation.momID': mom_id,
            u'ObsSW.Observation.startTime': future_start_time,
            u'ObsSW.Observation.ObservationControl.PythonControl.LongBaseline.subbands_per_subbandgroup': u'1',
            u'ObsSW.Observation.nrBeams': u'0',
            u'ObsSW.Observation.DataProducts.Input_IncoherentStokes.skip': [],
            u'ObsSW.Observation.ObservationControl.PythonControl.DPPP.demixer.demixfreqstep': u'64',
            u'ObsSW.Observation.DataProducts.Output_Correlated.enabled': True,
            u'ObsSW.Observation.sampleClock': u'200',
            u'ObsSW.Observation.processType': u'Pipeline',
            u'ObsSW.Observation.processSubtype': u'Averaging Pipeline',
        }

        self.observation_specification_tree = {
            u'ObsSW.Observation.DataProducts.Output_InstrumentModel.enabled': False,
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.nrChannelsPerSubband': u'64',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.timeIntegrationFactor': u'1',
            u'ObsSW.Observation.stopTime': future_stop_time,
            u'ObsSW.Observation.VirtualInstrument.stationList': [u'RS205', u'RS503', u'CS013', u'RS508', u'RS106'],
            u'ObsSW.Observation.DataProducts.Input_CoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Output_CoherentStokes.enabled': True,
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.Correlator.nrChannelsPerSubband': u'64',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.which': u'I',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.which': u'I',
            u'ObsSW.Observation.Beam[0].subbandList': [100, 101, 102, 103],
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.subbandsPerFile': u'512',
            u'ObsSW.Observation.DataProducts.Input_Correlated.skip': [],
            u'ObsSW.Observation.antennaSet': u'HBA_DUAL',
            u'ObsSW.Observation.nrBitsPerSample': u'8',
            u'ObsSW.Observation.Beam[0].nrTabRings': u'0',
            u'ObsSW.Observation.Beam[0].nrTiedArrayBeams': u'0',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.flysEye': False,
            u'ObsSW.Observation.nrBeams': u'1',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.Correlator.integrationTime': u'1.0',
            u'ObsSW.Observation.DataProducts.Output_IncoherentStokes.enabled': True,
            u'ObsSW.Observation.DataProducts.Input_IncoherentStokes.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_Correlated.enabled': False,
            u'ObsSW.Observation.DataProducts.Output_Pulsar.enabled': False,
            u'ObsSW.Observation.DataProducts.Input_CoherentStokes.skip': [],
            u'ObsSW.Observation.DataProducts.Output_SkyImage.enabled': False,
            u'Version.number': u'33774',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.nrChannelsPerSubband': u'64',
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.timeIntegrationFactor': u'1',
            u'ObsSW.Observation.momID': mom_id,
            u'ObsSW.Observation.startTime': future_start_time,
            u'ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.subbandsPerFile': u'512',
            u'ObsSW.Observation.DataProducts.Input_IncoherentStokes.skip': [],
            u'ObsSW.Observation.DataProducts.Output_Correlated.enabled': True,
            u'ObsSW.Observation.sampleClock': u'200',
            u'ObsSW.Observation.processType': u'Observation',
            u'ObsSW.Observation.processSubtype': u'Beam Observation',
        }

        self.test_specification = {
            u'Version.number': u'33774',            
            u'Observation.momID': mom_id,
            u'Observation.sampleClock': u'200',
            u'Observation.DataProducts.Output_Correlated.enabled': True,
            u'Observation.ObservationControl.OnlineControl.Cobalt.Correlator.nrChannelsPerSubband': u'64',
            u'Observation.ObservationControl.OnlineControl.Cobalt.Correlator.integrationTime': u'1.0',
            u'Observation.DataProducts.Output_CoherentStokes.enabled': True,
            u'Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.nrChannelsPerSubband': u'64',
            u'Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.CoherentStokes.timeIntegrationFactor': u'1',
            u'Observation.DataProducts.Output_IncoherentStokes.enabled': True,
            u'Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.nrChannelsPerSubband': u'64',
            u'Observation.ObservationControl.OnlineControl.Cobalt.BeamFormer.IncoherentStokes.timeIntegrationFactor': u'1',
        }
         
        self.test_cobalt_settings = {
            'blockSize': 196608, 'nrBlocks': 1, 'integrationTime': 1.00663296, 'nrSubblocks': 1
        }

        self.otdb_info = {
            'LOFAR.ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.Correlator.nrBlocksPerIntegration': 1,
            'LOFAR.ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.blockSize': 196608,
            'LOFAR.ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.Correlator.integrationTime': 1.00663296,
            'LOFAR.ObsSW.Observation.ObservationControl.OnlineControl.Cobalt.Correlator.nrIntegrationsPerBlock': 1
        }


    def setUp(self):
        #init
        self.mom_id = 351557
        self.otdb_id = 1290494
        self.trigger_id = 2323
        future_start_time = (datetime.datetime.utcnow() + datetime.timedelta(hours=1)).strftime('%Y-%m-%d %H:%M:%S')
        future_stop_time = (datetime.datetime.utcnow() + datetime.timedelta(hours=2)).strftime('%Y-%m-%d %H:%M:%S')
        self.modification_time = datetime.datetime.utcnow()
        
        self.reset_specification_tree(self.otdb_id, self.mom_id, future_start_time, future_stop_time)
        otdbrpc_patcher = mock.patch('lofar.sas.otdb.otdbrpc')
        self.addCleanup(otdbrpc_patcher.stop)
        self.otdbrpc_mock = otdbrpc_patcher.start()
        self.otdbrpc_mock.taskGetSpecification.return_value = {'otdb_id': self.otdb_id, 'specification': self.observation_specification_tree}
        self.otdbrpc_mock.setOTDBinfo.return_value = {}

        momrpc_patcher = mock.patch('lofar.mom.momqueryservice.momqueryrpc')
        self.addCleanup(momrpc_patcher.stop)
        self.momrpc_mock = momrpc_patcher.start()
        self.momrpc_mock.getMoMIdsForOTDBIds.return_value = {self.otdb_id: self.mom_id}
        self.momrpc_mock.get_trigger_id.return_value = {'status': 'OK', 'trigger_id': self.trigger_id}

        radbrpc_patcher = mock.patch('lofar.sas.resourceassignment.resourceassignmentservice.rpc')
        self.addCleanup(radbrpc_patcher.stop)
        self.radbrpc_mock = radbrpc_patcher.start()
      
        logger_patcher = mock.patch('lofar.sas.resourceassignment.taskprescheduler.taskprescheduler.logger')
        self.addCleanup(logger_patcher.stop)
        self.logger_mock = logger_patcher.start()
        
        self.taskprescheduler = TestingTaskPrescheduler(self.otdbrpc_mock, self.momrpc_mock, self.radbrpc_mock)

    def assert_all_services_opened(self):
        self.assertTrue(self.otdbrpc_mock.open.called, "OTDBRPC.open was not called")
        self.assertTrue(self.momrpc_mock.open.called, "MOMRPC.open was not called")
        self.assertTrue(self.radbrpc_mock.open.called, "RADBRPC.open was not called")

    def assert_all_services_closed(self):
        self.assertTrue(self.otdbrpc_mock.close.called, "OTDBRPC.close was not called")
        self.assertTrue(self.momrpc_mock.close.called, "MOMRPC.close was not called")
        self.assertTrue(self.radbrpc_mock.close.called, "RADBRPC.close was not called")

    @mock.patch("lofar.messaging.messagebus.AbstractBusListener.start_listening")
    def test_open_opens_all_services(self, mock_super):
        self.taskprescheduler.start_listening()
        self.assert_all_services_opened()
        self.assertTrue(mock_super.called)

    @mock.patch("lofar.messaging.messagebus.AbstractBusListener.stop_listening")
    def test_close_closes_all_services(self, mock_super):
        self.taskprescheduler.stop_listening()
        self.assert_all_services_closed()
        self.assertTrue(mock_super.called)
        
    def test_onObservationApproved_GetSpecification(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskGetSpecification.assert_any_call(otdb_id=self.otdb_id)

    def test_resourceIndicatorsFromParset(self):
        specification = resourceIndicatorsFromParset(self.observation_specification_tree)
        self.assertEqual(specification, self.test_specification)

    def test_CobaltOTDBsettings(self):
        otdb_info = cobaltOTDBsettings(self.test_cobalt_settings)
        self.assertEqual(otdb_info, self.otdb_info)

    def test_onObservationApproved_log_mom_id_found(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.info.assert_any_call('Found mom_id %s for otdb_id %s', self.mom_id, self.otdb_id)

    def test_onObservationApproved_log_mom_id_not_found(self):
        observation_specification_tree_no_momid = self.observation_specification_tree
        observation_specification_tree_no_momid[u'ObsSW.Observation.momID'] = u''
        self.otdbrpc_mock.taskGetSpecification.return_value = {'otdb_id': self.otdb_id,
                                                               'specification': observation_specification_tree_no_momid}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.info.assert_any_call('Did not find a mom_id for task otdb_id=%s', self.otdb_id)

    def test_onObservationApproved_otdb_specification_problem(self):
        self.otdbrpc_mock.taskGetSpecification.return_value = {'otdb_id': 0, 'specification': ''}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.exception.assert_called()
        self.logger_mock.error.assert_called()
        self.radbrpc_mock.updateTaskStatusForOtdbId.assert_any_call(self.otdb_id, 'error')

    def test_onObservationApproved_log_trigger_found(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.info.assert_any_call('Found a task mom_id=%s with a trigger_id=%s', self.mom_id, self.trigger_id)

    def test_onObservationApproved_log_no_trigger(self):
        self.momrpc_mock.get_trigger_id.return_value = {"trigger_id": None, "status": "Error", "errors": ["Trigger doesn''t exist"]}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.info.assert_any_call('Did not find a trigger for task mom_id=%s, because %s', self.mom_id, ["Trigger doesn''t exist"])

    def test_onObservationApproved_no_trigger(self):
        self.momrpc_mock.get_trigger_id.return_value = {"trigger_id": None, "status": "Error", "errors": ["Trigger doesn''t exist"]}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskSetSpecification.assert_not_called()
        self.otdbrpc_mock.taskSetStatus.assert_not_called()

    def test_onObservationApproved_log_trigger_found(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.logger_mock.info.assert_any_call('Setting status (%s) for otdb_id %s', 'prescheduled', self.otdb_id)

    def test_onObservationApproved_SetSpecification(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskSetSpecification.assert_any_call(self.otdb_id, self.otdb_info)

    def test_onObservationApproved_pipeline_SetSpecification(self):
        self.otdbrpc_mock.taskGetSpecification.return_value = {'otdb_id': self.otdb_id,
                                                               'specification': self.pipeline_specification_tree}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskSetSpecification.assert_not_called()

    def test_onObservationApproved_taskSetStatus(self):
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskSetStatus.assert_any_call(self.otdb_id, 'prescheduled')

    def test_setOTDBinfo_taskSetStatus(self):
        self.taskprescheduler.setOTDBinfo(self.otdb_id, self.otdb_info, 'prescheduled')
        self.otdbrpc_mock.taskSetStatus.assert_any_call(self.otdb_id, 'prescheduled')

    def test_setOTDBinfo_otdb_problem(self):
        self.otdbrpc_mock.taskSetSpecification.side_effect = Exception()
        self.taskprescheduler.setOTDBinfo(self.otdb_id, self.otdb_info, 'prescheduled')
        self.radbrpc_mock.updateTaskStatusForOtdbId.assert_any_call(self.otdb_id, 'error')

    def test_onObservationApproved_pipeline_taskSetStatus(self):
        self.otdbrpc_mock.taskGetSpecification.return_value = {'otdb_id': self.otdb_id,
                                                               'specification': self.pipeline_specification_tree}
        self.taskprescheduler.onObservationApproved(self.otdb_id, self.modification_time)
        self.otdbrpc_mock.taskSetStatus.assert_any_call(self.otdb_id, 'prescheduled')

    def test_calculateCobaltSettings(self):
        cobalt_settings = calculateCobaltSettings(self.test_specification)
        self.assertEqual(cobalt_settings, self.test_cobalt_settings)

    @mock.patch("lofar.common.util.waitForInterrupt")
    @mock.patch("lofar.sas.resourceassignment.taskprescheduler.taskprescheduler.TaskPrescheduler")
    def test_main(self, mock_wait, mock_prescheduler):
        PreschedulerMain()
        mock_wait.assert_called()

if __name__ == "__main__":
    unittest.main()
